# encoding: utf-8

"""
Learning rules to be used for unsupervised neural network learning.
2009, Maria Johansen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""


class LearningRule:
	def __init__(self, learningrate):
		self.learningrate=learningrate
	def update(self, prenode, postnode):
		return NotImplemented
		
class ClassicHebbian(LearningRule):
	def __init__(self, learningrate):
		LearningRule.__init__(self, learningrate)
	def update(self, prenode_activation_level, postnode_activation_level, weight):
		return self.learningrate*prenode_activation_level*postnode_activation_level
		
class GeneralHebbian(LearningRule):
	def __init__(self, learningrate, threshold=0.5):
		LearningRule.__init__(self, learningrate)
		self.threshold=threshold
	def update(self, prenode_activation_level, postnode_activation_level, weight):
		return self.learningrate*(prenode_activation_level-self.threshold)*(postnode_activation_level-self.threshold)

class Oja(LearningRule):
	def __init__(self, learningrate):
		LearningRule.__init__(self, learningrate)
	def update(self, prenode_activation_level, postnode_activation_level, weight):
		return self.learningrate*(prenode_activation_level*postnode_activation_level-weight*(postnode_activation_level**2))
