#!/usr/bin/env python
# encoding: utf-8

"""
2009, Maria Johansen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""

from epuck import *
from ann import *
from utilities import *
from behavior_layers import *

import os
import sys
import getopt
from multiprocessing import Process

import psyco; psyco.full()


#Default values:
EPUCK_ID = 1231
EPUCK_PATHS = "/dev/rfcomm0 /dev/rfcomm1 /dev/rfcomm2"
EPUCK_PATHS = "/dev/rfcomm0 /dev/rfcomm1"
EPUCK_PATHS = "/dev/rfcomm1"
GS_SETTINGS_PATH="goto_sound.ann"
OA_SETTINGS_PATH="obs_avoidance.ann"
GL_SETTINGS_PATH=None

def optproc():
	"""Read parameters from command line
	"""
	
	help_message = """
Usage:
        ./main.py [OPTIONS] 

Options:
        --oa Path to obstacle avoidance ann settings
        --gs Path to goto sound ann settings
        --gl Path to goto red led ann settings
        --epuck-paths Device names/ports that e-puck is connected to (separated by whitespace)
        --help Display this help message
"""
	
	try:
		try:
			longoptions = ["help", "gs=", "gl=", "oa=", "epuck-paths="]
			opts, args = getopt.getopt(sys.argv[1:], "h", longoptions)
		except getopt.error, msg:
			raise Exception(msg)
			
		gs=GS_SETTINGS_PATH
		oa=OA_SETTINGS_PATH
		gl=GL_SETTINGS_PATH
		epuck_paths=EPUCK_PATHS
			
		for option, value in opts:
			if option == '--gs':
				gs = value
			if option in ("-h", "--help"):
				raise Exception(help_message)
			if option == "--oa":
				oa = value
			if option == "--gl":
				gl = value
			if option == "--epuck-paths":
				epuck_paths = value
				
		epuck_paths=epuck_paths.split(' ')
		return oa,gs,gl,epuck_paths
		
	except Exception, e:
		print "Error: ", e
		print help_message
		sys.exit(1)

	
class DataSet:
	"""Simple class to keep track of ann training and testdata
	"""
	def __init__(self, trainingdata, testdata):
		self.trainingdata=trainingdata;
		self.testdata=testdata

def robot_train_oa1(ann):
	"""Delta learning training data for obstacle
	avoidance on e-puck.
	"""
	front=([300,0,0,0,0,0,0,0],[-1,0])
	right1=([0,300,0,0,0,0,0,0],[-1,0])
	right2=([0,0,300,0,0,0,0,0],[-1,0])
	
	delta_learning(ann, [front,right1,right2])
	
def robot_train_oa_old(ann):
	"""Delta learning training data for obstacle
	avoidance on e-puck.
	"""
	front1=([0,0,0,0,0,0,0,300],[0,-1])
	front2=([300,0,0,0,0,0,0,0],[-1,0])
	right1=([0,300,0,0,0,0,0,0],[-1,0])
	right2=([0,0,300,0,0,0,0,0],[-1,0])
	#right3=([0,0,0,300,0,0,0,0],[-1,0])
	left1=([0,0,0,0,0,0,300,0],[0,-1])
	left2=([0,0,0,0,0,300,0,0],[0,-1])
	#left3=([0,0,0,0,300,0,0,0],[0,-1])
	
	delta_learning(ann, [front1,front2,right1,right2,left1, left2])
	
def robot_train_oa2(ann):
	"""Delta learning training data for obstacle
	avoidance on e-puck.
	"""
	front=([0,0,0,0,0,0,0,300],[0,-1])
	left1=([0,0,0,0,0,0,300,0],[0,-1])
	left2=([0,0,0,0,0,300,0,0],[0,-1])
	
	delta_learning(ann, [front,left1,left2])

##test, -1,1	
def robot_train_oa_test(ann):
	"""Delta learning training data for obstacle
	avoidance on e-puck.
	"""
	front1=([0,0,0,0,0,0,0,300],[-1,1])
	front2=([300,0,0,0,0,0,0,0],[-1,1])
	right1=([0,300,0,0,0,0,0,0],[-1,1])
	right2=([0,0,300,0,0,0,0,0],[-1,1])
	#right3=([0,0,0,300,0,0,0,0],[-1,1])
	left1=([0,0,0,0,0,0,300,0],[1,-1])
	left2=([0,0,0,0,0,300,0,0],[1,-1])
	#left3=([0,0,0,0,300,0,0,0],[1,-1])
	
	delta_learning(ann, [front1,front2,right1,right2,left1, left2])
	
def robot_backpropagation_train(ann):
	"""Backpropagation training data for obstacle
	avoidance on e-puck.
	"""
	front1=([0,0,0,0,0,0,0,300],[1,0])
	front2=([300,0,0,0,0,0,0,0],[0,1])
	front3=([0,300,0,0,0,0,300,0],[0,0])
	rightleft1=([0,300,0,0,0,0,300,0],[0,0])
	rightleft1=([0,0,300,0,0,300,0,0],[0,0])
	right1=([0,300,0,0,0,0,0,0],[0,1])
	right2=([0,0,300,0,0,0,0,0],[0,1])
	#right3=([0,0,0,300,0,0,0,0],[-1,0])
	left1=([0,0,0,0,0,0,300,0],[1,0])
	left2=([0,0,0,0,0,300,0,0],[1,0])
	#left3=([0,0,0,0,300,0,0,0],[0,-1])
	
	backpropagation(ann, [front1, front2])


def new_epuck_process(epuck,settings_oa, settings_gs, settings_gl):
	"""Create a new process using multiprocessing to control the epuck.
	"""

	print "Starting epuck " + epuck.port + " ..."

	goto_sound_ann= create_ann(os.path.join(os.getcwd(), settings_gs))

	obs_avoidance_ann_old= create_ann(os.path.join(os.getcwd(), settings_oa))
	robot_train_oa_old(obs_avoidance_ann_old)
	
	obs_avoidance_ann_test= create_ann(os.path.join(os.getcwd(), settings_oa))
	robot_train_oa_test(obs_avoidance_ann_test)

	obs_avoidance_ann1= create_ann(os.path.join(os.getcwd(), settings_oa))
	robot_train_oa1(obs_avoidance_ann1)
	print obs_avoidance_ann1

	obs_avoidance_ann2= create_ann(os.path.join(os.getcwd(), settings_oa))
	robot_train_oa2(obs_avoidance_ann2)

	lock=threading.Lock()
	threaded=False

	layers=[]
	layers.append(Wander(epuck, None, 0.5,0.5, lock,threaded))
	#layers.append(AvoidObstacles(epuck, obs_avoidance_ann_old, lock,threaded))
	#layers.append(AvoidObstacles(epuck, obs_avoidance_ann1, lock,threaded))
	#layers.append(AvoidObstacles(epuck, obs_avoidance_ann2, lock,threaded))
	layers.append(AvoidObstacles(epuck, obs_avoidance_ann_test, lock,threaded))
	#layers.append(GotoSound(epuck, goto_sound_ann, lock,threaded))
	#layers.append(GotoLight(epuck, None, lock,threaded))
	#layers.append(GoHome(epuck, None, lock,threaded))
	try:
		epuck.move_wheels(0.5,0.5)
		B=BehaviorController(epuck, layers, lock,threaded)
		B.initialize()
	except:
		print "error: ", e
	finally:
		epuck.death()
		print "done"	



def main():
	#Use settings from command line (or defaults):
	settings_oa, settings_gs, settings_gl,epuck_paths=optproc() 
	epucks=[]
	

	#Create epucks and append them to epucks[]
	for path in epuck_paths:
		epuck=Epuck(port=path)
		epuck.birth()
		epucks.append(epuck)
		
	#Create one new process using multiprocessing for each epuck
	for epuck in epucks:
		Process(target=new_epuck_process, args=(epuck,settings_oa, settings_gs, settings_gl)).start()


	#obs_avoidance_ann1= create_ann(os.path.join(os.getcwd(), settings_oa))
	#robot_train_oa_test(obs_avoidance_ann1)
	#print obs_avoidance_ann1
	#obs_avoidance_ann1.test([[0,0,0,0,0,0,0,300],[0,3000,0,0,0,0,3000,0], [300,0,0,0,0,0,0,0]])
	#print obs_avoidance_ann_old


	
if __name__ == "__main__":
	main()


