#!/usr/bin/env python
# encoding: utf-8

"""
General utility functions
2009, Maria Johansen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""

import os
import sys
import pickle
import random
import itertools


def save_object(path, obj):
	"""Saves object to file, using the pickle module"""
	fp = open(path, 'w')
	pickle.dump(obj, fp) 

def load_object(path):
	"""Loads object from file. Must be saved in a format recognizable by the pickle module"""
	fp = open(path, 'r')
	obj=pickle.load(fp)
	return obj
	
def random_float(min,max):
	"""Return random float in range [min,max).
	"""
	return random.uniform(min,max)

def random_int(min,max):
	"""Return random int in range [min,max]
	"""
	return random.randint(min,max)

def itertools_flatten(listoflists):
	"""Returns a flattened version of a list of lists
	"""	
	return list(itertools.chain(*listoflists))
	
def nonzero(listoflists):
	"""Input is a list of lists of numbers. Return true if at least one of the
	elements is nonzero
	"""
	for e in itertools_flatten(listoflists):
		if e!=0:
			return True
	return False

def list_element_sum(listoflists):
	"""Assumes input is a list of equal length lists, return
	element-wise sum.
	"""
	summed=[]
	summed = [0.0]*len(listoflists[0])
	for l in listoflists:
		for i in range(len(l)):
			summed[i]+=l[i]
	return summed
	
def list_element_average(listoflists):
	"""Assumes input is a list of equal length lists, return
	element-wise averaged sum.
	"""
	summed=[]
	n=len(listoflists)
	summed = [0.0]*len(listoflists[0])
	for l in listoflists:
		for i in range(len(l)):
			summed[i]+=l[i]
	return [e/n for e in summed]
	
class CircularBuffer:
	"""Simple circular buffer class, with 'size' elements
	"""
	def __init__(self, size):
		self.data = []
		self.size=size
	def append(self, x):
		if len(self.data)>=self.size:
			self.data.pop(0)
		self.data.append(x)
	def get(self):
		return self.data

	
	
		
		
